@@ -33,10 +33,12 @@ ENVIRONMENT::ENVIRONMENT (GLOBALDATA *global)
         landSlideDelay = MAX_GRAVITY_DELAY;
 	windstrength = 8;
 	windvariation = 1;
-	techLevel = 5;
+	weapontechLevel = itemtechLevel = 5;
 	landType = LANDTYPE_RANDOM;
 	meteors = 0;
 	lightning = 0;
+        satellite = 0.0;
+        falling_dirt_balls = 0.0;
 	fog = 0;
         wallType = 0.0;
         current_wallType = 0;
@@ -65,75 +67,6 @@ ENVIRONMENT::ENVIRONMENT (GLOBALDATA *global)
 	initialise ();
 }
 
-// Following are config file definitions
-#define	GFILE_LATEST_VERSION	4
-
-// stop using wind strength as an int. Use the new double version
-#define GFILE_WINDSTRENGTH	&windstrength, sizeof (int), 1, file
-#define GFILE_NEW_WINDSTRENGTH  &windstrength, sizeof (double), 1, file
- 
-#define	GFILE_WINDVARIATION	&windvariation, sizeof (double), 1, file
-#define	GFILE_VISCOSITY		&viscosity, sizeof (double), 1, file
-#define	GFILE_GRAVITY		&gravity, sizeof (double), 1, file
-#define	GFILE_TECHLEVEL		&techLevel, sizeof (double), 1, file
-#define	GFILE_METEORS		&meteors, sizeof (double), 1, file
-#define	GFILE_LIGHTNING		&lightning, sizeof (double), 1, file
-#define	GFILE_FOG		&fog, sizeof (double), 1, file
-#define	GFILE_LANDTYPE		&landType, sizeof (double), 1, file
-#define	GFILE_LANDSLIDETYPE	&landSlideType, sizeof (double), 1, file
-#define GFILE_WALLTYPE          &wallType, sizeof(double), 1, file
-
-#define	GFILE_VERSION_1(a)	a (GFILE_WINDSTRENGTH);\
-				a (GFILE_WINDVARIATION);\
-				a (GFILE_VISCOSITY);\
-				a (GFILE_GRAVITY);\
-				a (GFILE_TECHLEVEL);\
-				a (GFILE_METEORS);\
-				a (GFILE_FOG);\
-				a (GFILE_LANDTYPE);\
-				a (GFILE_LANDSLIDETYPE);
-
-#define	GFILE_VERSION_2(a)	a (GFILE_WINDSTRENGTH);\
-				a (GFILE_WINDVARIATION);\
-				a (GFILE_VISCOSITY);\
-				a (GFILE_GRAVITY);\
-				a (GFILE_TECHLEVEL);\
-				a (GFILE_METEORS);\
-				a (GFILE_LIGHTNING);\
-				a (GFILE_FOG);\
-				a (GFILE_LANDTYPE);\
-				a (GFILE_LANDSLIDETYPE);
-
-#define GFILE_VERSION_3(a)      a (GFILE_WINDSTRENGTH);\
-                                a (GFILE_WINDVARIATION);\
-                                a (GFILE_VISCOSITY);\
-                                a (GFILE_GRAVITY);\
-                                a (GFILE_TECHLEVEL);\
-                                a (GFILE_METEORS);\
-                                a (GFILE_LIGHTNING);\
-                                a (GFILE_FOG);\
-                                a (GFILE_LANDTYPE);\
-                                a (GFILE_LANDSLIDETYPE);\
-                                a (GFILE_WALLTYPE);
-
-#define GFILE_VERSION_4(a)      a (GFILE_NEW_WINDSTRENGTH);\
-                                a (GFILE_WINDVARIATION);\
-                                a (GFILE_VISCOSITY);\
-                                a (GFILE_GRAVITY);\
-                                a (GFILE_TECHLEVEL);\
-                                a (GFILE_METEORS);\
-                                a (GFILE_LIGHTNING);\
-                                a (GFILE_FOG);\
-                                a (GFILE_LANDTYPE);\
-                                a (GFILE_LANDSLIDETYPE);\
-                                a (GFILE_WALLTYPE);
-
-
-#define	GFILE_WRITE_LATEST	GFILE_VERSION_4(fwrite)
-#define	GFILE_READ_VERSION_1	GFILE_VERSION_1(fread)
-#define	GFILE_READ_VERSION_2	GFILE_VERSION_2(fread)
-#define GFILE_READ_VERSION_3    GFILE_VERSION_3(fread)
-#define GFILE_READ_VERSION_4    GFILE_VERSION_4(fread)
 
 
 /*
@@ -150,17 +83,21 @@ int ENVIRONMENT::saveToFile_Text (FILE *file)
 
    fprintf(file, "*ENV*\n");
 
+   fprintf(file, "WINDSTRENGTH=%lf\n", windstrength);
    fprintf(file, "WINDVARIATION=%lf\n", windvariation);
    fprintf(file, "VISCOSITY=%lf\n", viscosity);
    fprintf(file, "GRAVITY=%lf\n", gravity);
-   fprintf(file, "TECHLEVEL=%lf\n", techLevel);
+   fprintf(file, "WEAPONTECHLEVEL=%lf\n", weapontechLevel);
+   fprintf(file, "ITEMTECHLEVEL=%lf\n", itemtechLevel);
    fprintf(file, "METEORS=%lf\n", meteors);
    fprintf(file, "LIGHTNING=%lf\n", lightning);
+   fprintf(file, "SATELLITE=%lf\n", satellite);
    fprintf(file, "FOG=%lf\n", fog);
    fprintf(file, "LANDTYPE=%lf\n", landType);
    fprintf(file, "LANDSLIDETYPE=%lf\n", landSlideType);
    fprintf(file, "WALLTYPE=%lf\n", wallType);
    fprintf(file, "LANDSLIDEDELAY=%lf\n", landSlideDelay);
+   fprintf(file, "FALLINGDIRTBALLS=%lf\n", falling_dirt_balls);
 
    fprintf(file, "***\n");      
    return TRUE;
@@ -168,18 +105,6 @@ int ENVIRONMENT::saveToFile_Text (FILE *file)
 
 
 
-int ENVIRONMENT::saveToFile (FILE *file)
-{
-	int version = GFILE_LATEST_VERSION;
-	if (!file)
-		return (-1);
-
-	fwrite (&version, sizeof (int), 1, file);
-
-	GFILE_WRITE_LATEST
-
-	return (ferror (file));
-}
 
 
 
@@ -237,18 +162,29 @@ int ENVIRONMENT::loadFromFile_Text (FILE *file)
              strncpy(field, line, equal_position);
              strcpy(value, & (line[equal_position + 1]));
              // check for fields and values
-             if (! strcasecmp(field, "windvariation") )
+             if (! strcasecmp(field, "windstrength") )
+                sscanf(value, "%lf", &windstrength);
+             else if (! strcasecmp(field, "windvariation") )
                 sscanf(value, "%lf", &windvariation);
              else if (! strcasecmp(field, "viscosity") )
                 sscanf(value, "%lf", &viscosity);
              else if (! strcasecmp(field, "gravity") )
                 sscanf(value, "%lf", &gravity);
              else if (! strcasecmp(field, "techlevel"))
-                sscanf(value, "%lf", &techLevel);
+             {
+                sscanf(value, "%lf", &weapontechLevel);
+                itemtechLevel = weapontechLevel;    // for backward compatibility
+             }
+             else if (! strcasecmp(field, "weapontechlevel") )
+                sscanf(value, "%lf", &weapontechLevel);
+             else if (! strcasecmp(field, "itemtechlevel") )
+                sscanf(value, "%lf", &itemtechLevel);
              else if (! strcasecmp(field, "meteors"))
                 sscanf(value, "%lf", &meteors);
              else if (! strcasecmp(field, "lightning") )
                 sscanf(value, "%lf", &lightning);
+             else if (! strcasecmp(field, "satellite") )
+                sscanf(value, "%lf", &satellite);
              else if (! strcasecmp(field, "fog") )
                 sscanf(value, "%lf", &fog);
              else if (! strcasecmp(field, "landtype"))
@@ -259,6 +195,9 @@ int ENVIRONMENT::loadFromFile_Text (FILE *file)
                 sscanf(value, "%lf", &wallType);
              else if (! strcasecmp(field, "landslidedelay"))
                 sscanf(value, "%lf", &landSlideDelay);
+             else if (! strcasecmp(field, "fallingdirtballs") )
+                sscanf(value, "%lf", &falling_dirt_balls);
+
           }    // end of found field=value line
 
       }     // end of read a line properly
@@ -268,33 +207,6 @@ int ENVIRONMENT::loadFromFile_Text (FILE *file)
 }
 
 
-int ENVIRONMENT::loadFromFile (FILE *file)
-{
-	int version;
-	if (!file)
-		return (-1);
-
-	fread (&version, sizeof (int), 1, file);
-
-	switch (version) {
-		case 1:
-			GFILE_READ_VERSION_1;
-			break;
-		case 2:
-			GFILE_READ_VERSION_2;
-			break;
-                case 3:
-                        GFILE_READ_VERSION_3;
-                        break;
-                case 4:
-                        GFILE_READ_VERSION_4;
-                        break;
-		default:
-			break;
-	}
-
-	return (ferror (file));
-}
 
 void ENVIRONMENT::initialise ()
 {
@@ -324,9 +236,9 @@ int ENVIRONMENT::isItemAvailable (int itemNum)
 {
 	if (itemNum < WEAPONS) {
 		if ((weapon[itemNum].warhead) ||
-			(weapon[itemNum].techLevel > techLevel))
+			(weapon[itemNum].techLevel > weapontechLevel))
 			return (FALSE);
-	} else if (item[itemNum - WEAPONS].techLevel > techLevel) {
+	} else if (item[itemNum - WEAPONS].techLevel > itemtechLevel) {
 		return (FALSE);
 	}
 	return (TRUE);
@@ -400,9 +312,10 @@ void ENVIRONMENT::newRound ()
 		}
 	}
 
+        naturals_since_last_shot = 0;
         // set wall type
         if (wallType == WALL_RANDOM)
-           current_wallType = rand() % 3;
+           current_wallType = rand() % 4;
         else
            current_wallType = (int) wallType;
 
@@ -475,6 +388,7 @@ void ENVIRONMENT::do_updates ()
 {
 	int count;
 
+        /*
 	if (fog && _global->currTank) {
 		int x, y;
 		x = (int)_global->currTank->x;
@@ -486,7 +400,7 @@ void ENVIRONMENT::do_updates ()
 		}
 		blit (db, screen, x - 100, y - 100, x - 100, y - 100, 200, 200);
 		blit (db, screen, 0, 0, 0, 0, _global->screenWidth, MENU);
-	} else {
+	} else { */
 		if (_global->updateCount == MAXUPDATES)
 			_global->updateCount = 1;
 		for (count = 0; count < _global->updateCount && count < MAXUPDATES; count++) {
@@ -494,7 +408,7 @@ void ENVIRONMENT::do_updates ()
 			// Debug rectangle
 			//rect (screen, _global->updates[count].x, _global->updates[count].y, _global->updates[count].x + _global->updates[count].w, _global->updates[count].y + _global->updates[count].h, WHITE);
 		}
-	}
+	// }
 	_global->lastUpdatesCount = _global->updateCount;
 	memcpy (_global->lastUpdates, _global->updates, sizeof (BOX) * _global->updateCount);
 	_global->updateCount = 0;
