@@ -31,10 +31,12 @@ enum landscapeTypes {
 	LANDTYPE_HILLS,
 	LANDTYPE_FOOTHILLS,
 	LANDTYPE_PLAIN,
+        LANDTYPE_NONE
 };
 
 enum landSlideTypes {
 	LANDSLIDE_NONE,       // gravity does not exist
+        LANDSLIDE_TANK_ONLY,  // dirt falls, tank does not
 	LANDSLIDE_INSTANT,    // dirt falls without you seeing it
 	LANDSLIDE_GRAVITY,    // normal
         LANDSLIDE_CARTOON     // gravity is delayed
@@ -49,12 +51,25 @@ enum wallTypes {
         WALL_RUBBER,
 	WALL_STEEL,
 	WALL_SPRING,
+        WALL_WRAP,
         WALL_RANDOM
 };
 
 
+// stages
+#define STAGE_AIM 0
+#define STAGE_ENDGAME 3
+
+
 #define SPRING_CHANGE 1.25
 
+// size of satellite laser
+#define LASER_NONE 0.0
+#define LASER_WEAK 1.0
+#define LASER_STRONG 2.0
+#define LASER_SUPER 3.0
+
+
 class TANK;
 class MISSILE;
 class FLOATTEXT;
@@ -74,9 +89,10 @@ class ENVIRONMENT {
 	double	windstrength, windvariation;
 	double	viscosity;
 	double	gravity;
-	double	techLevel;
+	double	weapontechLevel, itemtechLevel;
 	double	meteors;
 	double	lightning;
+        double  falling_dirt_balls;
 	double	fog;
 	int	oldFogX, oldFogY;
 	double	landType;
@@ -97,6 +113,8 @@ class ENVIRONMENT {
         double  wallType;
         int     current_wallType, wallColour;
         int     time_to_fall;    // amount of time dirt will hover
+        double  satellite;
+        int     naturals_since_last_shot;
 
 	ENVIRONMENT () {};
 	ENVIRONMENT (GLOBALDATA *global);
@@ -114,8 +132,6 @@ class ENVIRONMENT {
 	void make_update (int x, int y, int w, int h);
 	void make_bgupdate (int x, int y, int w, int h);
 
-	int saveToFile (FILE *file);
-	int loadFromFile (FILE *file);
         int saveToFile_Text (FILE *file);
         int loadFromFile_Text (FILE *file);
 };
