@@ -25,6 +25,7 @@
 #include "tank.h"
 #include "player.h"
 
+
 EXPLOSION::EXPLOSION ()
 {
 	_align = LEFT;
@@ -58,6 +59,13 @@ EXPLOSION::EXPLOSION (GLOBALDATA *global, ENVIRONMENT *env, double xpos, double
 	etime = weap->etime;
 	damage = weap->damage;
 	eframes = weap->eframes;
+
+        // make sure dirt appears on the screen, not above the playing area
+        if ((type >= DIRT_BALL) && (type <= SUP_DIRT_BALL))
+        {
+           if (y < DIRT_CEILING)
+              y = DIRT_CEILING + radius;
+        }
 }
 
 EXPLOSION::EXPLOSION (GLOBALDATA *global, ENVIRONMENT *env, double xpos, double ypos, double xvel, double yvel, int weaponType)
@@ -212,7 +220,7 @@ void EXPLOSION::explode ()
 			tmp = create_bitmap(radius * 2, radius * 2);
 			clear_to_color(tmp, PINK);
 			for (int count = 0; count < radius ; count++) {
-				circle (tmp, radius, radius, count, (player)?player->color:WHITE);
+				circle (tmp, radius, radius, count, (player)?player->color: makecol (0, 255, 0) );
 			}
 			setUpdateArea ((int)x - (radius + 1), (int)y - (radius + 1), (radius + 1) * 2, (radius + 1) * 2);
 
@@ -249,7 +257,7 @@ void EXPLOSION::draw (BITMAP *dest)
 			rotate_scaled_sprite (dest, (BITMAP *) _global->gfxData.explosions[(a + (EXPLOSIONFRAMES * weap->eframes)) - 2], (int)x - radius, (int)y - radius, itofix (0), ftofix ((double) radius / 107));
 			setUpdateArea ((int)x - (radius + 1), (int)y - (radius + 1), (radius + 1) * 2, (radius + 1) * 2);
 		}
-	} else if (type <= TECTONIC) {
+	} else if ( (type <= TECTONIC) && (type >= TREMOR) ) {
 		if (a > 1 && a <= EXPLODEFRAMES + 1) {
 			drawFracture (_global, _env, _env->terrain, &_current, (int)x, (int)y, angle, (int)(((double)a / EXPLODEFRAMES) * (radius / 4)), radius, 5, 0);
 		}
@@ -271,7 +279,7 @@ void EXPLOSION::draw (BITMAP *dest)
 	} else {
 		if (a > 1 && a <= EXPLODEFRAMES + 1) {
 			int startCirc = (radius / EXPLODEFRAMES) * a;
-			circlefill (dest, (int)x, (int)y, startCirc, (player)?player->color:WHITE);
+			circlefill (dest, (int)x, (int)y, startCirc, (player)?player->color: makecol (0, 255, 0) );
 			startCirc += (radius / EXPLODEFRAMES) * 2;
 			setUpdateArea ((int)x - startCirc, (int)y - startCirc, startCirc * 2, startCirc * 2);
 		}
