@@ -160,8 +160,8 @@ changeHandleError
 
 Used internally by change and quickChange.  Prints out error messages.
 *****************************************************************************/
-static void changeHandleError (BITMAP *target) {
-	show_mouse (NULL);
+static void changeHandleError (GLOBALDATA *global, BITMAP *target) {
+	if (! global->os_mouse) show_mouse (NULL);
 	if (errorMessage) { 	 
 		textout_ex (target, font, errorMessage, errorX, errorY, makecol (255, 0, 0), -1); 	 
 		errorMessage = NULL; 	 
@@ -184,8 +184,8 @@ void change(GLOBALDATA *global, BITMAP *target)
 	} ;
 	static const int cprocs = sizeof( procs ) / sizeof( procs[0] );
 
-	show_mouse (NULL);
-	changeHandleError (target);
+	if (! global->os_mouse) show_mouse (NULL);
+	changeHandleError (global, target);
 
 	(procs[rand() % cprocs]) (global, target);
 
@@ -200,7 +200,7 @@ Executes a fast and simple transition.
 
 *****************************************************************************/
 void quickChange (GLOBALDATA *global, BITMAP *target) {
-	changeHandleError (target);
+	changeHandleError (global, target);
 	blit (target, screen, 0, 0, 0, 0, global->screenWidth, global->screenHeight);
 	rest (1);
 }
