@@ -9,6 +9,7 @@
 #define ENVIRONMENT_STAGE 2
 #define PLAYER_STAGE 3
 
+#include <dirent.h>
 #include "globaldata.h"
 #include "environment.h"
 #include "lineseq.h"
@@ -25,6 +26,12 @@ int Load_Game(GLOBALDATA *global, ENVIRONMENT *env);
 
 int Check_For_Saved_Game(GLOBALDATA *global);
 
+/*
+Copy the atanks config file from HOME_DIR to
+HOME_DIR/.atanks
+*/
+int Copy_Config_File(GLOBALDATA *global);
+
 
 void renderTextLines (GLOBALDATA *global, ENVIRONMENT *env,
                 const LINESEQ& lines, int scrollOffset,
@@ -39,5 +46,11 @@ void drawMenuBackground (GLOBALDATA *global, ENVIRONMENT *env, int itemType, int
 
 void flush_inputs();
 
+int Load_Weapons_Text(GLOBALDATA *global);
+
+int Filter_File( const struct dirent *my_file );
+
+struct dirent ** Find_Saved_Games(GLOBALDATA *global, int *num_files_found);
+
 #endif
 
