@@ -31,12 +31,12 @@ class FLOATTEXT: public VIRTUAL_OBJECT
 
 	public:
 
-	FLOATTEXT (GLOBALDATA *global, ENVIRONMENT *env, const char *text, int x, int y, int color, alignType alignment);
+	FLOATTEXT (GLOBALDATA *global, ENVIRONMENT *env, char *text, int x, int y, int color, alignType alignment);
 	~FLOATTEXT ();
 	void		initialise ();
 	int		applyPhysics ();
 	void		draw (BITMAP *dest);
-	void		set_text (const char *text);
+	void		set_text (char *text);
 	void		set_pos (int x, int y);
 	void		set_color (int color);
 	int		isSubClass (int classNum);
