@@ -36,6 +36,8 @@ GLOBALDATA::GLOBALDATA ()
 	halfWidth = screenWidth / 2;
 	halfHeight = screenHeight / 2;
         frames_per_second = FRAMES_PER_SECOND;
+        numPermanentPlayers = 10;
+        violent_death = FALSE;
         #ifndef DOS
 	cacheCirclesBG = 1;
         #endif
@@ -62,6 +64,9 @@ GLOBALDATA::GLOBALDATA ()
 	sellpercent = 0.80;
         game_name[0] = '\0';
         load_game = 0.0;
+        campaign_mode = 0.0;
+        saved_game_index = 0;
+        saved_game_list = NULL;
 
 	updates = new BOX[MAXUPDATES];
 	if (!updates) {
@@ -95,136 +100,9 @@ GLOBALDATA::GLOBALDATA ()
 		slope[count][1] = cos (count / (180 / PI));
 	}
 	slope[270][1] = 0;
+        configDir = NULL;
 }
 
-// Following are config file definitions
-#define	GFILE_LATEST_VERSION	6
-
-#define GFILE_NUMPLAYERS	&numPlayers, sizeof (int), 1, file
-#define	GFILE_OLDROUNDS		&rounds, sizeof (int), 1, file
-#define	GFILE_ROUNDS		&rounds, sizeof (double), 1, file
-#define	GFILE_DITHER		&ditherGradients, sizeof (double), 1, file
-#define	GFILE_DETAILEDSKY	&detailedSky, sizeof (double), 1, file
-#define	GFILE_DETAILEDLAND	&detailedLandscape, sizeof (double), 1, file
-#define	GFILE_STARTMONEY	&startmoney, sizeof (double), 1, file
-#define	GFILE_TURNTYPE		&turntype, sizeof (double), 1, file
-#define	GFILE_INTEREST		&interest, sizeof (double), 1, file
-#define	GFILE_SCOREROUNDWINBONUS	&scoreRoundWinBonus, sizeof (double), 1, file
-#define	GFILE_SCOREHITUNIT		&scoreHitUnit, sizeof (double), 1, file
-#define	GFILE_SCORESELFHIT		&scoreSelfHit, sizeof (double), 1, file
-#define	GFILE_SCOREUNITDESTROYBONUS	&scoreUnitDestroyBonus, sizeof (double), 1, file
-#define	GFILE_SCOREUNITSELFDESTROY	&scoreUnitSelfDestroy, sizeof (double), 1, file
-#define	GFILE_ACCELERATEDAI	&skipComputerPlay, sizeof (double), 1, file
-#define GFILE_SELLPERCENT	&sellpercent, sizeof(double), 1, file
-#define GFILE_ENABLESOUND       &sound, sizeof(double), 1, file
-#define GFILE_SCREENWIDTH       &screenWidth, sizeof(int), 1, file
-#define GFILE_SCREENHEIGHT      &screenHeight, sizeof(int), 1, file
-#define GFILE_OS_MOUSE          &os_mouse, sizeof(double), 1, file
-
-
-#define	GFILE_VERSION_1(a)	a (GFILE_NUMPLAYERS);\
-				a (GFILE_OLDROUNDS);\
-				a (GFILE_DITHER);\
-				a (GFILE_DETAILEDSKY);\
-				a (GFILE_DETAILEDLAND);\
-				a (GFILE_STARTMONEY);\
-				a (GFILE_TURNTYPE);\
-				a (GFILE_INTEREST);
-
-#define	GFILE_VERSION_2(a)	a (GFILE_NUMPLAYERS);\
-				a (GFILE_ROUNDS);\
-				a (GFILE_DITHER);\
-				a (GFILE_DETAILEDSKY);\
-				a (GFILE_DETAILEDLAND);\
-				a (GFILE_STARTMONEY);\
-				a (GFILE_TURNTYPE);\
-				a (GFILE_INTEREST);\
-				a (GFILE_SCOREROUNDWINBONUS);\
-				a (GFILE_SCOREHITUNIT);\
-				a (GFILE_SCORESELFHIT);\
-				a (GFILE_SCOREUNITDESTROYBONUS);\
-				a (GFILE_SCOREUNITSELFDESTROY);
-
-#define	GFILE_VERSION_3(a)	a (GFILE_NUMPLAYERS);\
-				a (GFILE_ROUNDS);\
-				a (GFILE_DITHER);\
-				a (GFILE_DETAILEDSKY);\
-				a (GFILE_DETAILEDLAND);\
-				a (GFILE_STARTMONEY);\
-				a (GFILE_TURNTYPE);\
-				a (GFILE_INTEREST);\
-				a (GFILE_SCOREROUNDWINBONUS);\
-				a (GFILE_SCOREHITUNIT);\
-				a (GFILE_SCORESELFHIT);\
-				a (GFILE_SCOREUNITDESTROYBONUS);\
-				a (GFILE_SCOREUNITSELFDESTROY);\
-				a (GFILE_ACCELERATEDAI);
-
-#define	GFILE_VERSION_4(a)	a (GFILE_NUMPLAYERS);\
-				a (GFILE_ROUNDS);\
-				a (GFILE_DITHER);\
-				a (GFILE_DETAILEDSKY);\
-				a (GFILE_DETAILEDLAND);\
-				a (GFILE_STARTMONEY);\
-				a (GFILE_TURNTYPE);\
-				a (GFILE_INTEREST);\
-				a (GFILE_SCOREROUNDWINBONUS);\
-				a (GFILE_SCOREHITUNIT);\
-				a (GFILE_SCORESELFHIT);\
-				a (GFILE_SCOREUNITDESTROYBONUS);\
-				a (GFILE_SCOREUNITSELFDESTROY);\
-				a (GFILE_ACCELERATEDAI);\
-				a (GFILE_SELLPERCENT);
-
-#define GFILE_VERSION_5(a)      a (GFILE_NUMPLAYERS);\
-                                a (GFILE_ROUNDS);\
-                                a (GFILE_DITHER);\
-                                a (GFILE_DETAILEDSKY);\
-                                a (GFILE_DETAILEDLAND);\
-                                a (GFILE_STARTMONEY);\
-                                a (GFILE_TURNTYPE);\
-                                a (GFILE_INTEREST);\
-                                a (GFILE_SCOREROUNDWINBONUS);\
-                                a (GFILE_SCOREHITUNIT);\
-                                a (GFILE_SCORESELFHIT);\
-                                a (GFILE_SCOREUNITDESTROYBONUS);\
-                                a (GFILE_SCOREUNITSELFDESTROY);\
-                                a (GFILE_ACCELERATEDAI);\
-                                a (GFILE_SELLPERCENT);\
-                                a (GFILE_ENABLESOUND);\
-                                a (GFILE_SCREENWIDTH);\
-                                a (GFILE_SCREENHEIGHT);
-
-#define GFILE_VERSION_6(a)      a (GFILE_NUMPLAYERS);\
-                                a (GFILE_ROUNDS);\
-                                a (GFILE_DITHER);\
-                                a (GFILE_DETAILEDSKY);\
-                                a (GFILE_DETAILEDLAND);\
-                                a (GFILE_STARTMONEY);\
-                                a (GFILE_TURNTYPE);\
-                                a (GFILE_INTEREST);\
-                                a (GFILE_SCOREROUNDWINBONUS);\
-                                a (GFILE_SCOREHITUNIT);\
-                                a (GFILE_SCORESELFHIT);\
-                                a (GFILE_SCOREUNITDESTROYBONUS);\
-                                a (GFILE_SCOREUNITSELFDESTROY);\
-                                a (GFILE_ACCELERATEDAI);\
-                                a (GFILE_SELLPERCENT);\
-                                a (GFILE_ENABLESOUND);\
-                                a (GFILE_SCREENWIDTH);\
-                                a (GFILE_SCREENHEIGHT);\
-                                a (GFILE_OS_MOUSE);
-
-
-
-#define	GFILE_WRITE_LATEST	GFILE_VERSION_6(fwrite)
-#define	GFILE_READ_VERSION_1	GFILE_VERSION_1(fread)
-#define	GFILE_READ_VERSION_2	GFILE_VERSION_2(fread)
-#define	GFILE_READ_VERSION_3	GFILE_VERSION_3(fread)
-#define	GFILE_READ_VERSION_4	GFILE_VERSION_4(fread)
-#define GFILE_READ_VERSION_5    GFILE_VERSION_5(fread)
-#define GFILE_READ_VERSION_6    GFILE_VERSION_6(fread)
-
 
 /*
 This function saves the global data to a text file. If all goes
@@ -263,6 +141,7 @@ int GLOBALDATA::saveToFile_Text( FILE *file)
    fprintf(file, "LANGUAGE=%lf\n", language);
    fprintf(file, "COLOURTHEME=%lf\n", colour_theme);
    fprintf(file, "FRAMES=%lf\n", frames_per_second);
+   fprintf(file, "VIOLENTDEATH=%lf\n", violent_death);
    fprintf(file, "***\n");
    return TRUE;
 }
@@ -371,6 +250,8 @@ int GLOBALDATA::loadFromFile_Text (FILE *file)
                 sscanf(value, "%lf", &colour_theme);
              else if (! strcasecmp(field, "frames") )
                 sscanf(value, "%lf", &frames_per_second);
+             else if (! strcasecmp(field, "violentdeath") )
+                sscanf(value, "%lf", &violent_death);
          }    // end of found field=value line
 
       }     // end of read a line properly
@@ -395,82 +276,6 @@ int GLOBALDATA::loadFromFile_Text (FILE *file)
 
 
 
-int GLOBALDATA::saveToFile (FILE *file)
-{
-	int version = GFILE_LATEST_VERSION;
-	if (!file)
-		return (-1);
-
-        screenWidth = (int) temp_screenWidth;
-        screenHeight = (int) temp_screenHeight;
-
-	fwrite (&version, sizeof (int), 1, file);
-
-	GFILE_WRITE_LATEST
-
-	return (ferror (file));
-}
-
-
-/*
-This function loads the global options from the config file.
-A note on the sound thing going on: Sound is ON by default,
-however, before we get here, Atanks checks the command line
-to see if we should turn sound OFF. So, if sound is OFF we
-we get here, we save that over-ride from the command line.
-However, if the sound is ON when we get here, we leave it
-ON.
--- Jesse
-*/
-int GLOBALDATA::loadFromFile (FILE *file)
-{
-	int version;
-        double sound_bookmark = 1.0;
-
-        if (! sound)
-            sound_bookmark = sound;
-
-	if (!file)
-		return (-1);
-
-	fread (&version, sizeof (int), 1, file);
-
-	switch (version) {
-		case 1:
-			GFILE_READ_VERSION_1;
-			break;
-		case 2:
-			GFILE_READ_VERSION_2;
-			break;
-		case 3:
-			GFILE_READ_VERSION_3;
-			break;
-		case 4:
-			GFILE_READ_VERSION_4;
-			break;
-                case 5:
-                        GFILE_READ_VERSION_5;
-                        break;
-                case 6:
-                        GFILE_READ_VERSION_6;
-                        break;
-		default:
-			break;
-	}
-
-        if (! sound_bookmark)
-           sound = sound_bookmark;
-
-        if (width_override)
-           screenWidth = width_override;
-        if (height_override)
-           screenHeight = height_override;
-
-        halfWidth = screenWidth / 2;
-        halfHeight = screenHeight / 2;
-
-	return (ferror (file));
-}
 
 void GLOBALDATA::initialise ()
 {
@@ -553,3 +358,27 @@ void GLOBALDATA::destroyPlayer (PLAYER *player)
 	}
 	numPermanentPlayers--;
 }
+
+
+
+// This function returns the path to the
+// config directory used by Atanks
+char *GLOBALDATA::Get_Config_Path()
+{
+     char *my_config_dir;
+     char *homedir;
+
+     // figure out file name
+     homedir = getenv(HOME_DIR);
+     if (! homedir)
+        homedir = ".";
+     my_config_dir = (char *) calloc( strlen(homedir) + 24,
+                                      sizeof(char) );
+     if (! my_config_dir)
+        return NULL;
+
+     sprintf(my_config_dir, "%s/.atanks", homedir);
+     return my_config_dir;
+
+}
+
