@@ -27,12 +27,19 @@
 
 #define LANGUAGE_ENGLISH 0.0
 #define LANGUAGE_PORTUGUESE 1.0
+#define LANGUAGE_FRENCH 2.0
 
 #define COLOUR_THEME_REGULAR 0.0
 #define COLOUR_THEME_CRISPY 1.0
 
 #define GAME_NAME_LENGTH 64
 
+#define VIOLENT_DEATH_OFF 0
+#define VIOLENT_DEATH_LIGHT 1
+#define VIOLENT_DEATH_MEDIUM 2
+#define VIOLENT_DEATH_HEAVY 3
+
+
 enum skipComputerPlayType {
 	SKIP_NONE, SKIP_HUMANS_DEAD// , SKIP_AUTOPLAY
 };
@@ -45,6 +52,7 @@ class GLOBALDATA {
 	double	slope[360][2];
 
 	char	*dataDir;
+        char    *configDir;
 	BOX	*updates, *lastUpdates, window;
 	int	updateCount, lastUpdatesCount;
 	int	stopwindow;
@@ -115,11 +123,13 @@ class GLOBALDATA {
         char    tank_status[128];
         char    game_name[GAME_NAME_LENGTH];
         double  load_game;
+        double  campaign_mode;
+        double  violent_death;
+        double  saved_game_index;
+        char    **saved_game_list;
 
 	GLOBALDATA ();
 	void	initialise ();
-	int	saveToFile (FILE *file);
-	int	loadFromFile (FILE *file);
         int     saveToFile_Text (FILE *file);
         int     loadFromFile_Text (FILE *file);
 	void	addPlayer (PLAYER *player);
@@ -127,6 +137,7 @@ class GLOBALDATA {
 	PLAYER	*getNextPlayer (int *playerCount);
 	PLAYER	*createNewPlayer (ENVIRONMENT *env);
 	void	destroyPlayer (PLAYER *player);
+        char    *Get_Config_Path();
 };
 
 #endif
