@@ -30,7 +30,6 @@
 #include "player.h"
 #include "environment.h"
 #include "globaldata.h"
-#include "weapons.h"
 #include "teleport.h"
 #include "decor.h"
 #include "beam.h"
@@ -397,6 +396,7 @@ const gradient * const land_gradients[] = {
 int		cclock, 
 		fps, frames,
 		fi, lx, ly;
+#define CLOCK_MAX 10
 
 //char		cacheCirclesBG = 1;
 //char		ditherGradients = 1;
@@ -419,5 +419,10 @@ char buf[100];
 
 bool quit_right_now;
 
+WEAPON  weapon[WEAPONS];
+WEAPON  naturals[NATURALS];
+ITEM    item[ITEMS];
+
+
 #endif
 
