@@ -21,10 +21,9 @@
  * */
 
 #ifndef VERSION
-#define VERSION "2.6"
+#define VERSION "3.0"
 #endif
 
-
 #include <allegro.h>
 #ifdef WIN32
 #include <winalleg.h>
@@ -56,6 +55,14 @@
 
 using namespace std;
 
+// place to save config and save games
+#ifdef WIN32
+#define HOME_DIR "AppData"
+#endif
+#ifdef LINUX
+#define HOME_DIR "HOME"
+#endif
+
 #ifndef	DATA_DIR
 #define	DATA_DIR	"."
 #endif
@@ -83,10 +90,10 @@ using namespace std;
 #define LANDS 8
 #define ALL_SKIES 16
 #define ALL_LANDS 16
-#define	BALLISTICS 42
+#define	BALLISTICS 45
 #define	BEAMWEAPONS 3
 #define WEAPONS (BALLISTICS + BEAMWEAPONS)
-#define ITEMS 20
+#define ITEMS 21
 #define THINGS (WEAPONS + ITEMS)
 #define	NATURALS 6
 #define RADII 6
@@ -128,7 +135,11 @@ using namespace std;
 #define BACKGROUND_LINE 1
 #define BACKGROUND_BLANK 2
 
-#define WAIT_AT_END_OF_ROUND 60
+#define WAIT_AT_END_OF_ROUND 100
+
+// time between volly shots
+#define VOLLY_DELAY 50
+
 
 // defines for teams
 #define TEAM_SITH 0
@@ -163,8 +174,8 @@ typedef struct gradient_struct {
 class WEAPON
 {
 	public:
-	char		*name;		//name of weapon
-	char		*description;
+	char		name[128];		//name of weapon
+	char		description[256];
 	int             cost;		//$ :))
 	int		amt;		//number of weapons in one package
 	double		mass;
@@ -179,7 +190,7 @@ class WEAPON
 	int		delay;		// volleys etc.
 	int		noimpact;
 	int		techLevel;
-	char		warhead;	// Is it a warhead?
+	int		warhead;	// Is it a warhead?
 	int		numSubmunitions;// Number of submunitions
 	int		submunition;	// The next stage
 	double		impartVelocity;	// Impart velocity 0.0-1.0 to subs
@@ -201,8 +212,8 @@ class WEAPON
 class ITEM
 {
 	public:
-	char		*name;
-	char		*description;
+	char		name[128];
+	char		description[256];
 	int		cost;
 	int		amt;
 	int		selectable;
@@ -210,6 +221,8 @@ class ITEM
 	int		sound;
 	double		vals[MAX_ITEMVALS];
 };
+
+
 enum shieldVals {
 	SHIELD_ENERGY,
 	SHIELD_REPULSION,
@@ -226,8 +239,11 @@ enum selfDestructVals {
 enum weaponType {
 	SML_MIS, MED_MIS, LRG_MIS, SML_NUKE, NUKE, DTH_HEAD,
 	SML_SPREAD, MED_SPREAD, LRG_SPREAD, SUP_SPREAD, DTH_SPREAD, ARMAGEDDON,
+        CHAIN_MISSILE,
 	SHAPED_CHARGE, WIDE_BOY, CUTTER,
 	SML_ROLLER, LRG_ROLLER, DTH_ROLLER,
+        SMALL_MIRV,
+        ARMOUR_PIERCING,
 	CLUSTER, SUP_CLUSTER,
 	FUNKY_BOMB, FUNKY_DEATH,
 	FUNKY_BOMBLET, FUNKY_DEATHLET,
@@ -267,7 +283,8 @@ enum itemType {
 	ITEM_SLICKP,
 	ITEM_DIMPLEP,
 	ITEM_PARACHUTE,
-        ITEM_REPAIRKIT
+        ITEM_REPAIRKIT,
+        ITEM_FUEL
 };
 
 #define SHIELD_COUNT		6
@@ -345,6 +362,8 @@ void generate_sky (GLOBALDATA *global, BITMAP* bmp, const gradient* grad, int fl
 
 // save all game settings
 int Save_Game_Settings(GLOBALDATA *global, ENVIRONMENT *env, char *path_to_file);
+int Save_Game_Settings_Text(GLOBALDATA *global, ENVIRONMENT *env, char *path_to_file);
+
 // handle changes to global settings
 int Change_Settings(double old_mouse, double old_sound, double new_mouse, double new_sound, void *mouse_image);
 
