@@ -27,6 +27,8 @@
 #define MAX_MISSLE_AGE 20
 #define MAX_METEOR_AGE 5
 
+#define TIGGER_HEIGHT 300
+
 class MISSILE: public PHYSICAL_OBJECT
 {
 	public:
@@ -51,6 +53,7 @@ class MISSILE: public PHYSICAL_OBJECT
 	void	initialise ();
 	int	isSubClass (int classNum);
 	int	getClass () {return (MISSILE_CLASS);};
+        int     Height_Above_Ground ();
 };
 
 #endif
