@@ -45,7 +45,6 @@ PLAYER::PLAYER (GLOBALDATA *global, ENVIRONMENT *env)
 	selected = FALSE;
         changed_weapon = false;
 
-	_targetMatrix = new double[global->screenWidth];
 
         tank_bitmap = 0.0;
 
@@ -81,7 +80,7 @@ PLAYER::PLAYER (GLOBALDATA *global, ENVIRONMENT *env)
 	typeText[0] = "Human";
 	typeText[1] = "Useless";
 	typeText[2] = "Guesser";
-	typeText[3] = "Range Finder";
+	typeText[3] = "Range";
 	typeText[4] = "Targetter";
 	typeText[5] = "Deadly";
 	preftypeText[0] = "Per Game";
@@ -108,7 +107,6 @@ PLAYER::PLAYER (GLOBALDATA *global, ENVIRONMENT *env, FILE *file, bool text_file
         tank_bitmap = 0.0;
         team = TEAM_NEUTRAL;
 
-	_targetMatrix = new double[global->screenWidth];
 
 	nm[0] = 99;
 	for (int count = 1; count < WEAPONS; count++)
@@ -126,15 +124,13 @@ PLAYER::PLAYER (GLOBALDATA *global, ENVIRONMENT *env, FILE *file, bool text_file
 
 	if (text_file)
            loadFromFile_Text (file);
-        else
-           loadFromFile (file);
 
 	type = (int)type;
 
 	typeText[0] = "Human";
 	typeText[1] = "Useless";
 	typeText[2] = "Guesser";
-	typeText[3] = "Range Finder";
+	typeText[3] = "Ranger";
 	typeText[4] = "Targetter";
 	typeText[5] = "Deadly";
 	preftypeText[0] = "Per Game";
@@ -218,6 +214,9 @@ void PLAYER::initMenuDesc ()
 		exit (1);
 	}
 
+        //init memory
+        memset(menuopts, 0, menudesc->numEntries * sizeof(MENUENTRY));
+
 	// Player name
 	menuopts[i].name = "Name";
 	menuopts[i].displayFunc = NULL;
@@ -355,166 +354,6 @@ PLAYER::~PLAYER ()
 	delete menudesc;
 }
 
-// Following are config file definitions
-#define	GFILE_LATEST_VERSION	11
-
-#define GFILE_NAME		_name, sizeof (char), 11, file
-#define GFILE_NAME_EXTENDED     _name, sizeof (char), NAME_LENGTH, file
-#define	GFILE_WEAPONPREFERENCE1	_weaponPreference, sizeof (int), 51, file
-#define	GFILE_WEAPONPREFERENCE2	_weaponPreference, sizeof (int), 54, file
-#define	GFILE_WEAPONPREFERENCE3	_weaponPreference, sizeof (int), 58, file
-#define	GFILE_WEAPONPREFERENCE4	_weaponPreference, sizeof (int), 60, file
-#define	GFILE_WEAPONPREFERENCE5	_weaponPreference, sizeof (int), 62, file
-#define	GFILE_TYPE		&type, sizeof (double), 1, file
-#define	GFILE_VENGEFUL		&vengeful, sizeof (int), 1, file
-#define	GFILE_VENGEANCETHRESHOLD &vengeanceThreshold, sizeof (double), 1, file
-#define	GFILE_COLOR		&color, sizeof (int), 1, file
-#define	GFILE_COLOR2		&color2, sizeof (int), 1, file
-#define	GFILE_PLAYED		&played, sizeof (double), 1, file
-#define	GFILE_WON		&won, sizeof (double), 1, file
-#define GFILE_PREFTYPE		&preftype, sizeof (double), 1, file
-#define	GFILE_SELFPRESERVATION	&selfPreservation, sizeof (double), 1, file
-#define	GFILE_PAINSENSITIVITY	&painSensitivity, sizeof (double), 1, file
-#define	GFILE_DEFENSIVE		&defensive, sizeof (double), 1, file
-#define GFILE_TANK_BITMAP       &tank_bitmap, sizeof (double), 1, file
-#define GFILE_TURRET_BITMAP    &turret_bitmap, sizeof (double), 1, file
-
-#define	GFILE_VERSION_1(a)	a (GFILE_NAME);\
-				a (GFILE_TYPE);\
-				a (GFILE_COLOR);\
-				a (GFILE_WEAPONPREFERENCE1);\
-				a (GFILE_PLAYED);\
-				a (GFILE_WON);
-
-#define	GFILE_VERSION_2(a)	a (GFILE_NAME);\
-				a (GFILE_TYPE);\
-				a (GFILE_COLOR);\
-				a (GFILE_COLOR2);\
-				a (GFILE_WEAPONPREFERENCE2);\
-				a (GFILE_PLAYED);\
-				a (GFILE_WON);
-
-#define	GFILE_VERSION_3(a)	a (GFILE_NAME);\
-				a (GFILE_TYPE);\
-				a (GFILE_COLOR);\
-				a (GFILE_COLOR2);\
-				a (GFILE_WEAPONPREFERENCE3);\
-				a (GFILE_PLAYED);\
-				a (GFILE_WON);
-
-#define	GFILE_VERSION_4(a)	a (GFILE_NAME);\
-				a (GFILE_TYPE);\
-				a (GFILE_COLOR);\
-				a (GFILE_COLOR2);\
-				a (GFILE_WEAPONPREFERENCE4);\
-				a (GFILE_PLAYED);\
-				a (GFILE_WON);
-
-#define	GFILE_VERSION_5(a)	a (GFILE_NAME);\
-				a (GFILE_VENGEFUL);\
-				a (GFILE_VENGEANCETHRESHOLD);\
-				a (GFILE_TYPE);\
-				a (GFILE_COLOR);\
-				a (GFILE_COLOR2);\
-				a (GFILE_WEAPONPREFERENCE4);\
-				a (GFILE_PLAYED);\
-				a (GFILE_WON);
-
-#define	GFILE_VERSION_6(a)	a (GFILE_NAME);\
-				a (GFILE_VENGEFUL);\
-				a (GFILE_VENGEANCETHRESHOLD);\
-				a (GFILE_TYPE);\
-				a (GFILE_COLOR);\
-				a (GFILE_COLOR2);\
-				a (GFILE_WEAPONPREFERENCE4);\
-				a (GFILE_PLAYED);\
-				a (GFILE_WON);\
-				a (GFILE_PREFTYPE);
-
-#define	GFILE_VERSION_7(a)	a (GFILE_NAME);\
-				a (GFILE_VENGEFUL);\
-				a (GFILE_VENGEANCETHRESHOLD);\
-				a (GFILE_TYPE);\
-				a (GFILE_COLOR);\
-				a (GFILE_COLOR2);\
-				a (GFILE_WEAPONPREFERENCE4);\
-				a (GFILE_PLAYED);\
-				a (GFILE_WON);\
-				a (GFILE_PREFTYPE);\
-				a (GFILE_SELFPRESERVATION);\
-				a (GFILE_PAINSENSITIVITY);
-
-#define	GFILE_VERSION_8(a)	a (GFILE_NAME);\
-				a (GFILE_VENGEFUL);\
-				a (GFILE_VENGEANCETHRESHOLD);\
-				a (GFILE_TYPE);\
-				a (GFILE_COLOR);\
-				a (GFILE_COLOR2);\
-				a (GFILE_WEAPONPREFERENCE5);\
-				a (GFILE_PLAYED);\
-				a (GFILE_WON);\
-				a (GFILE_PREFTYPE);\
-				a (GFILE_SELFPRESERVATION);\
-				a (GFILE_PAINSENSITIVITY);
-
-#define	GFILE_VERSION_9(a)	a (GFILE_NAME);\
-				a (GFILE_VENGEFUL);\
-				a (GFILE_VENGEANCETHRESHOLD);\
-				a (GFILE_TYPE);\
-				a (GFILE_COLOR);\
-				a (GFILE_COLOR2);\
-				a (GFILE_WEAPONPREFERENCE5);\
-				a (GFILE_PLAYED);\
-				a (GFILE_WON);\
-				a (GFILE_PREFTYPE);\
-				a (GFILE_SELFPRESERVATION);\
-				a (GFILE_PAINSENSITIVITY);\
-				a (GFILE_DEFENSIVE);
-
-#define GFILE_VERSION_10(a)     a (GFILE_NAME_EXTENDED);\
-                                a (GFILE_VENGEFUL);\
-                                a (GFILE_VENGEANCETHRESHOLD);\
-                                a (GFILE_TYPE);\
-                                a (GFILE_COLOR);\
-                                a (GFILE_COLOR2);\
-                                a (GFILE_WEAPONPREFERENCE5);\
-                                a (GFILE_PLAYED);\
-                                a (GFILE_WON);\
-                                a (GFILE_PREFTYPE);\
-                                a (GFILE_SELFPRESERVATION);\
-                                a (GFILE_PAINSENSITIVITY);\
-                                a (GFILE_DEFENSIVE);
-
-#define GFILE_VERSION_11(a)     a (GFILE_NAME_EXTENDED);\
-                                a (GFILE_VENGEFUL);\
-                                a (GFILE_VENGEANCETHRESHOLD);\
-                                a (GFILE_TYPE);\
-                                a (GFILE_COLOR);\
-                                a (GFILE_COLOR2);\
-                                a (GFILE_WEAPONPREFERENCE5);\
-                                a (GFILE_PLAYED);\
-                                a (GFILE_WON);\
-                                a (GFILE_PREFTYPE);\
-                                a (GFILE_SELFPRESERVATION);\
-                                a (GFILE_PAINSENSITIVITY);\
-                                a (GFILE_DEFENSIVE);\
-                                a (GFILE_TANK_BITMAP);\
-                                a (GFILE_TURRET_BITMAP); // remove this line in version 12
-
-
-
-#define	GFILE_WRITE_LATEST	GFILE_VERSION_11(fwrite)
-#define	GFILE_READ_VERSION_1	GFILE_VERSION_1(fread)
-#define	GFILE_READ_VERSION_2	GFILE_VERSION_2(fread)
-#define	GFILE_READ_VERSION_3	GFILE_VERSION_3(fread)
-#define	GFILE_READ_VERSION_4	GFILE_VERSION_4(fread)
-#define	GFILE_READ_VERSION_5	GFILE_VERSION_5(fread)
-#define GFILE_READ_VERSION_6	GFILE_VERSION_6(fread)
-#define GFILE_READ_VERSION_7	GFILE_VERSION_7(fread)
-#define GFILE_READ_VERSION_8	GFILE_VERSION_8(fread)
-#define GFILE_READ_VERSION_9	GFILE_VERSION_9(fread)
-#define GFILE_READ_VERSION_10   GFILE_VERSION_10(fread);
-#define GFILE_READ_VERSION_11   GFILE_VERSION_11(fread);
 
 
 
@@ -536,6 +375,7 @@ int PLAYER::saveToFile_Text (FILE *file)
    fprintf(file, "VENGEFUL=%d\n", vengeful);
    fprintf(file, "VENGEANCETHRESHOLD=%lf\n", vengeanceThreshold);
    fprintf(file, "TYPE=%lf\n", type);
+   fprintf(file, "TYPESAVED=%lf\n", type_saved);
    fprintf(file, "COLOR=%d\n", color);
    fprintf(file, "COLOR2=%d\n", color2);
    for (count = 0; count < THINGS; count++)
@@ -578,8 +418,8 @@ int PLAYER::loadFromFile_Text (FILE *file)
       result = fgets(line, MAX_CONFIG_LINE, file);
       if (! result)     // eof
          return FALSE;
-      if (! strncmp(line, "***", 3) )     // end of record
-         return FALSE;
+      // if (! strncmp(line, "***", 3) )     // end of record
+      //   return FALSE;
     } while ( strncmp(line, "*PLAYER*", 8) );     // read until we hit new record
 
     while ( (result) && (!done) )
@@ -619,6 +459,12 @@ int PLAYER::loadFromFile_Text (FILE *file)
                  sscanf(value, "%lf", &vengeanceThreshold);
              else if (! strcasecmp(field, "type") )
                  sscanf(value, "%lf", &type);
+             else if (! strcasecmp(field, "typesaved") )
+             {
+                 sscanf(value, "%lf", &type_saved);
+                 if (type_saved > HUMAN_PLAYER)
+                    type = type_saved;
+             }
              else if (! strcasecmp(field, "color") )
                  sscanf(value, "%d", &color);
              else if (! strcasecmp(field, "color2") )
@@ -658,99 +504,7 @@ int PLAYER::loadFromFile_Text (FILE *file)
 }
 
 
-int PLAYER::saveToFile (FILE *file)
-{
-	int version = GFILE_LATEST_VERSION;
-        double turret_bitmap;  // quick hack to fix version 11
 
-	if (!file)
-		return (-1);
-
-        // make sure part time bots are saved as human
-        if (type == PART_TIME_BOT)
-           type = HUMAN_PLAYER;
-
-	fwrite (&version, sizeof (int), 1, file);
-
-	GFILE_WRITE_LATEST
-
-	return (ferror (file));
-}
-
-int PLAYER::loadFromFile (FILE *file)
-{
-	int version;
-        double turret_bitmap;     // quick hack to fix version 11
-
-	if (!file)
-		return (-1);
-
-	fread (&version, sizeof (int), 1, file);
-
-	switch (version) {
-		case 1:
-			GFILE_READ_VERSION_1;
-			break;
-		case 2:
-			GFILE_READ_VERSION_2;
-			break;
-		case 3:
-			GFILE_READ_VERSION_3;
-			break;
-		case 4:
-			GFILE_READ_VERSION_4;
-			break;
-		case 5:
-			GFILE_READ_VERSION_5;
-			break;
-		case 6:
-			GFILE_READ_VERSION_6;
-			break;
-		case 7:
-			GFILE_READ_VERSION_7;
-			break;
-		case 8:
-			GFILE_READ_VERSION_8;
-			break;
-		case 9:
-			GFILE_READ_VERSION_9;
-			break;
-                case 10:
-                        GFILE_READ_VERSION_10;
-                        break;
-                case 11:
-                        GFILE_READ_VERSION_11;
-                        break;
-		default:
-			break;
-	}
-	
-	if (version < 6) {
-		// Apply change between v 5 and 6 of the player config
-		// 10% of time set to perplay weapon preferences
-		preftype = (rand () % 10)?ALWAYS_PREF:PERPLAY_PREF;
-		generatePreferences ();
-	}
-	if (version < 7) {
-		// Apply change between v 6 and 7
-		selfPreservation = (double)(rand () % 3000) / 1000;
-		painSensitivity = (double)(rand () % 3000) / 1000;
-	}
-	if (version < 8) {
-		// Apply changes between 7 and 8
-		generatePreferences ();
-	}
-	if (version < 9) {
-		// Apply changes between 8 and 9
-		defensive = (double)((rand () % 10000) - 5000) / 5000.0;
-	}
-
-        // make sure previous human players are restored as humans
-        if (type == PART_TIME_BOT)
-           type = HUMAN_PLAYER;
-
-	return (ferror (file));
-}
 
 void PLAYER::exitShop ()
 {
@@ -782,19 +536,17 @@ void PLAYER::newRound ()
 	}
 	tank->newRound ();
 
-        /*
-        This used to make sure money did not over-flow.
-        But the check has been moved to the buystuff function
-        because interest is applied there. 
-        -- Jesse
-        if (money > 1000000000)
-           money = 1000000000;
-        if (money < 0)
-           money = 0;
-        */
         changed_weapon = false;
+        // if we are playing in a campaign, raise the AI level
+        if (_global->campaign_mode)
+        {
+            if ( (type > HUMAN_PLAYER) && (type < DEADLY_PLAYER) )
+              type += 1.0;
+        }
+        
 }
 
+
 void PLAYER::initialise ()
 {
 	long int totalPrefs;
@@ -826,6 +578,8 @@ void PLAYER::initialise ()
 	}
 	while (rouletteCount < NUM_ROULETTE_SLOTS)
 		_rouletteWheel[rouletteCount++] = rand () % THINGS;
+
+        kills = killed = 0;
 }
 
 void PLAYER::generatePreferences ()
@@ -867,7 +621,7 @@ int PLAYER::controlTank (TANK *ctank)
 
 	if (_global->computerPlayersOnly &&
 		((int)_global->skipComputerPlay >= SKIP_HUMANS_DEAD)) {
-		if (_env->stage == 3)
+		if (_env->stage == STAGE_ENDGAME)
 			return (-1);
 	}
 
@@ -875,7 +629,7 @@ int PLAYER::controlTank (TANK *ctank)
 	if (keypressed () && !fi) {
 		k = readkey ();
 
-		if ((_env->stage == 3) &&
+		if ((_env->stage == STAGE_ENDGAME) &&
 			(k >> 8 == KEY_ENTER ||
 			 k >> 8 == KEY_ESC ||
 			 k >> 8 == KEY_SPACE))
@@ -928,7 +682,7 @@ int PLAYER::controlTank (TANK *ctank)
 	}
 	if ((int)type == HUMAN_PLAYER || !ctank) {
 		return (humanControls (ctank));
-	} else if (!_env->stage) {
+	} else if (_env->stage == STAGE_AIM) {
 		return (computerControls (ctank));
 	}
 	return (0);
@@ -1045,6 +799,8 @@ char *PLAYER::selectRevengePhrase (double scale)
            strcat(filename, "/revenge.txt");
         else if (_global->language == LANGUAGE_PORTUGUESE)
            strcat(filename, "/revenge.pt_BR.txt");
+        else if (_global->language == LANGUAGE_FRENCH)
+           strcat(filename, "/revenge_fr.txt");
 
         // static const LINESEQ says( filename, LINESEQ::skip_blanks );
         line = Random_Line(filename);
@@ -1066,6 +822,8 @@ char *PLAYER::selectGloatPhrase (double scale)
            strcat(filename, "/gloat.txt");
         else if (_global->language == LANGUAGE_PORTUGUESE)
            strcat(filename, "/gloat.pt_BR.txt");
+        else if (_global->language == LANGUAGE_FRENCH)
+           strcat(filename, "/gloat_fr.txt");
 
         // static const LINESEQ says( filename, LINESEQ::skip_blanks );
         line = Random_Line(filename);
@@ -1087,6 +845,8 @@ char *PLAYER::selectSuicidePhrase (double scale)
            strcat(filename, "/suicide.txt");
         else if (_global->language == LANGUAGE_PORTUGUESE)
            strcat(filename, "/suicide.pt_BR.txt");
+        else if (_global->language == LANGUAGE_FRENCH)
+           strcat(filename, "/suicide_fr.txt");
 
         // static const LINESEQ says( filename, LINESEQ::skip_blanks );
         line = Random_Line(filename);       
@@ -1096,7 +856,8 @@ char *PLAYER::selectSuicidePhrase (double scale)
 }
 
 
-int PLAYER::traceShellTrajectory (TANK *ctank, int *estimateX, int *estimateY, double targetX, double targetY, double x, double y, double xv, double yv)
+int PLAYER::traceShellTrajectory (TANK *ctank, int *estimateX, int *estimateY, double targetX, double targetY, 
+                                  double x, double y, double xv, double yv)
 {
 	TANK *ltank;
 	int hitSomething = 0;
@@ -1145,19 +906,35 @@ int PLAYER::traceShellTrajectory (TANK *ctank, int *estimateX, int *estimateY, d
                           hitSomething = 1;
                           break;
                       }
+                      else if ( _env->current_wallType == WALL_WRAP )
+                      {
+                          if (xv < 0)
+                             x = _global->screenWidth - 1;
+                          else 
+                             x = 1;
+                      }
 		} else {
 			// motion - wind affected
 			double accel = (_env->wind - xv) / weapon[ctank->cw].mass * drag * _env->viscosity;
 			xv += accel;
 			x += xv;
 		}
-		if (y + yv >= _global->screenHeight)
+		if (y + yv >= _global->screenHeight) 
 		{
+                    if ( _env->current_wallType == WALL_RUBBER )
+                    {
 			yv = -yv * 0.5;
 			xv *= 0.95;
+                    }
+                    else if ( _env->current_wallType == WALL_SPRING )
+                       yv = -yv * SPRING_CHANGE;
+                    else     // steel or wrap
+                       hitSomething = 1;
+            
 			if (fabs(xv) + fabs(yv) < 0.8)
 				hitSomething = 1;
 		}
+
 		y += yv;
 		yv += _env->gravity * (100.0 / _global->frames_per_second);
 
@@ -1194,6 +971,7 @@ int PLAYER::traceShellTrajectory (TANK *ctank, int *estimateX, int *estimateY, d
 		distance = -distance;
 	else if ((targetX - x > 0) && (direction > 0))
 		distance = -distance;
+
 	return ((int)distance);
 }
 
@@ -1284,6 +1062,13 @@ int PLAYER::calculatePowerGivenAngle (int dx, int dy, int angle)
 	double powerMult;
 	double minYv, minV;
 
+        if (tank)
+        {
+            if ( (tank->cw >= WEAPONS) || 
+               ( (tank->cw >= SML_LAZER) && (tank->cw <= LRG_LAZER) ) )
+               return _targetPower;
+        }
+
 	slopeX = _global->slope[angle][1];
 	slopeY = _global->slope[angle][0];
 
@@ -1363,286 +1148,73 @@ int PLAYER::adjustAngleGivenClearance (TANK *ctank, int xdist, int ydist, int cl
 	return (_targetAngle);
 }
 
-/*
- * Add the estimated scores for a given tank with a given weapon
- *   to _targetMatrix
- * Take into account how much damage is likely to be done and whether it will
- *   be enough to destroy the tank.
- * Positional correction to account for wind effect on 2nd stage.
- */
-int PLAYER::addTankDamageToMatrix (TANK *ctank, TANK *ltank, int weapNum)
-{
-	int radius = weapon[weapNum].radius;
-	double wdamage = calcTotalEffectiveDamage (weapNum) *
-					weapon[weapNum].spread;
-	int tankX = (int)ltank->x;
-	int tankY = (int)ltank->y;
-	double windAffect;
-
-	if ((weapNum >= RIOT_BOMB &&
-		 weapNum <= HVY_RIOT_BOMB)) {
-		// wdamage should probably be renamed to something
-		//   more accurate
-		wdamage = calcDefenseValue (ctank, ltank) *
-			radius * (defensive + 1);
-	}
-
-	// Adjust tanks pos according to drag on submunition
-	if (weapon[weapNum].submunition >= 0) {
-		windAffect = (weapon[weapon[weapNum].submunition].drag * _env->wind / _env->gravity);
-	} else {
-		windAffect = 0;
-	}
-
-	if ( (ltank->player == this) ||
-             ( (ltank->player->team == team) &&
-               (team != TEAM_NEUTRAL) ) )
-        {
-		wdamage = -wdamage * painSensitivity;
-		radius = (int)(radius * selfPreservation);
-	} else if (ltank->player == revenge) {
-		wdamage *= 1 + (double)vengeful / 50.0;
-	}
-
-	for (int cx = 0; cx < _global->screenWidth; cx++) {
-		// Weapon specific calculations...
-		int damage = 0;
-		double distance = 0.0;   // just to avoid compiler warnings
-		int cy = _env->surface[cx];
-		int effectiveTankX = (int)(tankX + ((tankY - cy) * windAffect));
-		if (weapNum >= RIOT_CHARGE && weapNum <= RIOT_BLAST) {
-			// add points only within range and above ctank
-			if (	(cy > ctank->y - (radius / 2)) ||
-				(fabs (ctank->x - cx) > radius))
-				continue;
-		} else if (weapNum >= SML_ROLLER && weapNum <= DTH_ROLLER) {
-			// Only aim rollers above other tanks
-			if (cy > tankY + TANKHEIGHT &&
-				fabs (ctank->x - cx) > radius)
-				continue;
-		} else if (weapNum >= SML_LAZER && weapNum <= LRG_LAZER) {
-			// Lazer can only be aimed above horizontal
-			if (cy > ctank->y)
-				continue;
-		} else if (weapNum >= SHAPED_CHARGE && weapNum <= CUTTER) {
-			double ydist = abs ((cy - tankY) * 20);
-			if (ydist > TANKHEIGHT/2)
-				ydist -= TANKHEIGHT/2;
-			else
-				ydist = 0;
-			distance = sqrt (pow ((float)cx - effectiveTankX, 2) + pow (ydist, 2));
-		}
-
-		if (weapNum < SHAPED_CHARGE || weapNum > CUTTER) {
-			distance = sqrt (pow ((float)cx - effectiveTankX, 2) + pow ((float)cy - tankY, 2));
-		}
-
-		// Estimate fall damage
-		if (cy - tankY > 0 && abs (cx - effectiveTankX) < radius) {
-			if (ltank->player == this)
-				damage -= radius - abs (cx - effectiveTankX);
-			else
-				damage += radius - abs (cx - effectiveTankX);
-		}
-
-		if (distance <= (radius + TANKHEIGHT/2)) {
-
-			damage += (int) (wdamage * (1.0 - (distance / (radius + TANKHEIGHT/2)) / 2));
-		}
-
-		// Getting rid of a tank is tactically advantageous as 
-		//   it is one less to shoot you.
-		// This could probably use a personality affector
-		//   (extent to which a player would rather finish a tank
-		//   off than cause up to x amount of damage total).
-		if (ltank->l + ltank->sh < damage) {
-			if (ltank->player == this)
-				damage -= 50;
-			else
-				damage += 50;
-		}
-
-		_targetMatrix[cx] += damage;
-	}
-	return (0);
-}
 
 
-/*
- * Calculate the damage matrix for the given weapon
- */
-void PLAYER::calcDamageMatrix (TANK *ctank, int weapNum)
-{
-	int objCount = 0;
-	TANK *ltank = NULL;
-
-	for (int cx = 0; cx < _global->screenWidth; cx++)
-		_targetMatrix[cx] = 0;
-
-	while (((ltank = (TANK*)_env->getNextOfClass (TANK_CLASS, &objCount))) || ltank) {
-		if (ltank->l <= 0)
-			continue;
-		if (weapNum < WEAPONS) {
-			addTankDamageToMatrix (ctank, ltank, weapNum);
-		} else {
-			int itemNum = weapNum - WEAPONS;
-			if (itemNum  >= ITEM_VENGEANCE && itemNum <= ITEM_FATAL_FURY) {
-				// add sqrt distances for each tank * potential damage
-				long int totalEffectiveDamage = calcTotalEffectiveDamage (itemNum);
-				_targetMatrix[(int)ctank->x] += sqrt (fabs (ctank->x - ltank->x)) * totalEffectiveDamage;
-
-			}
-		}
-		objCount++;
-	}
-}
 
 double PLAYER::selectTarget ()
 {
 	int targetXCoord, targetYCoord;
 
-	return (selectTarget (&targetXCoord, &targetYCoord));
+	// return (selectTarget (&targetXCoord, &targetYCoord));
+        return ( Select_Target( &targetXCoord, &targetYCoord) );
 }
 
-double PLAYER::selectTarget (int *targetXCoord, int *targetYCoord)
-{
-	double highestScore = 0;
-	int startAtX = rand () % _global->screenWidth;
 
-	*targetXCoord = 0;
-	for (int xCount = 0; xCount < _global->screenWidth; xCount++) {
-		int xPos = (startAtX + xCount) % _global->screenWidth;
-		if (_targetMatrix[xPos] > highestScore) {
-			*targetXCoord = xPos;
-			highestScore = _targetMatrix[xPos];
-		}
-	}
-	*targetYCoord = _env->surface[*targetXCoord];
-
-#ifdef	AI_PLANNING_DEBUG
-	// Plot damage matrix for debugging purposes
-	for (int xCount = 0; xCount < _global->screenWidth; xCount++) {
-		vline (screen, xCount, _env->surface[xCount], _env->surface[xCount] + 20, makecol ((int)(_targetMatrix[xCount] / highestScore * 200), (int)(_targetMatrix[xCount] / highestScore * 200), (int)(_targetMatrix[xCount] / highestScore * 200)));
-	}
-#endif
-
-	return (highestScore);
-}
-
-/*
- * calculate defense versus offense.
- * This can be based on turns to kill as a ratio (self to enemy).
- * If the ratio is too far in favour of the enemy, be defensive.
- * The ratio will largely be based on the armour of each tank as
- *   it's the only thing we know for sure. Perhaps later we could
- *   use known characteristics for the given player (fave weapon,
- *   defensiveness, accuracy etc.).
- */
-double PLAYER::calcDefenseValue (TANK *ctank, TANK *ltank)
-{
-	// double strengthRatio;
-	double armourRatio = 1.0;    // set a default, any default
-
-	if (ctank->l > 0 && ltank->l > 0) {
-		if (ctank->l + ctank->sh <= ltank->l + ltank->sh) {
-			armourRatio = (double)(ctank->l + ctank->sh) /
-					(double)(ltank->l + ltank->sh);
-		} else {
-			armourRatio = -(double)(ltank->l + ltank->sh) /
-					(double)(ctank->l + ctank->sh);
-		}
-	}
-
-	// strengthRatio = armourRatio;
-	// return (strengthRatio);
-        return armourRatio;
-}
 
 int PLAYER::computerSelectItem (TANK *ctank)
 {
-	int cw = 0; // Current Weapon
+	int current_weapon = 0; // Current Weapon
+	int targetXCoord, targetYCoord;
 
-	if (ctank->howBuried () > 135) {
+	if (ctank->howBuried () > BURIED_LEVEL) {
 		if (nm[RIOT_BLAST] > 0)
-			cw = RIOT_BLAST;
+			current_weapon = RIOT_BLAST;
 		else if (nm[RIOT_CHARGE] > 0)
-			cw = RIOT_CHARGE;
+			current_weapon = RIOT_CHARGE;
 		else if (nm[RIOT_BOMB] > 0)
-			cw = RIOT_BOMB;
+			current_weapon = RIOT_BOMB;
 		else if (nm[HVY_RIOT_BOMB] > 0)
-			cw = HVY_RIOT_BOMB;
+			current_weapon = HVY_RIOT_BOMB;
 		else if (ni[ITEM_TELEPORT] > 0)
-			cw = WEAPONS + ITEM_TELEPORT;
+			current_weapon = WEAPONS + ITEM_TELEPORT;
 		else if (nm[SML_LAZER] > 0)
-			cw = SML_LAZER;
+			current_weapon = SML_LAZER;
 		else if (nm[MED_LAZER] > 0)
-			cw = MED_LAZER;
+			current_weapon = MED_LAZER;
 		else if (nm[LRG_LAZER] > 0)
-			cw = LRG_LAZER;
-	}
-	if (cw == 0) {
-		double highestScore = -1;
-		int highestScorer = 0;
-		int targetXCoord, targetYCoord;
-		for (cw = 0; cw < THINGS; cw++) {
-
-			if (cw < WEAPONS) {
-				if (!nm[cw]) {
-					continue;
-				} else {
-					double score;
-					calcDamageMatrix (ctank, cw);
-					score = selectTarget (&targetXCoord,
-							&targetYCoord);
-					if (weapon[cw].cost > 0)
-						score = (score *
-							_weaponPreference[cw]) /
-							weapon[cw].cost;
-					if (score > highestScore) {
-						highestScorer = cw;
-						highestScore = score;
-						_targetX = targetXCoord;
-						_targetY = targetYCoord;
-					}
-				}
-			} else {
-				double score = 0.0;   // to avoid warnings
-				int itemNum = cw - WEAPONS;
-
-				if ((!ni[itemNum]) ||
-					(!item[itemNum].selectable))
-					continue;
-
-				if (itemNum == ITEM_TELEPORT) {
-					if (!ni[ITEM_PARACHUTE])
-						continue;
-				}
-
-				if ((itemNum >= ITEM_VENGEANCE) &&
-					(itemNum <= ITEM_FATAL_FURY)) {
-					if (ctank->l + ctank->sh < 20) {
-						calcDamageMatrix (ctank, cw);
-						score = selectTarget ();
-						score /= item[itemNum].cost;
-					}
-				}
-				if (score > highestScore) {
-					highestScorer = cw;
-					highestScore = score;
-				}
-			}
-			//break;
-		}
-		cw = highestScorer;
+			current_weapon = LRG_LAZER;
 	}
 
-	if ((cw < WEAPONS && !nm[cw]) || (cw >= THINGS) || (!ni[cw - WEAPONS]))
-		cw = 0;
+        current_weapon = Select_Random_Weapon();
+        // do not use riot items when not buried
+        if ( (current_weapon == RIOT_CHARGE) || (current_weapon == RIOT_BLAST) )
+        {
+           if ( ctank->howBuried() < BURIED_LEVEL )
+             current_weapon = 0;
+        }
+
+        if (! current_weapon)
+           current_weapon = Select_Random_Item();
+
+	if ((current_weapon < WEAPONS && !nm[current_weapon]) || 
+            (current_weapon >= THINGS) || (!ni[current_weapon - WEAPONS]))
+		current_weapon = 0;
+
+        ctank->cw = current_weapon;
+        // calcDamageMatrix (ctank, current_weapon);
+        // selectTarget (&targetXCoord, &targetYCoord);
+        Select_Target(& targetXCoord, &targetYCoord);
+	_targetX = targetXCoord;
+	_targetY = targetYCoord;
 
-	ctank->cw = cw;
 	_global->updateMenu = 1;
 
-	return (cw);
+        // avoid trying to shoot below the tank's level with lasers
+        if ( (_targetY > ctank->y) &&
+             ( (current_weapon >= SML_LAZER) && (current_weapon <= LRG_LAZER) ) )
+            ctank->cw = current_weapon = 0;
+
+	return (current_weapon);
 }
 
 int PLAYER::computerControls (TANK *ctank)
@@ -1654,9 +1226,12 @@ int PLAYER::computerControls (TANK *ctank)
 		_turnStage++;
 
 	} else if (_turnStage == SELECT_TARGET) {
-		selectTarget ();
+                // selectTarget ();
 		_turnStage++;
 
+        } else if ( (_turnStage == CALCULATE_ATTACK) && ( ctank->cw >= WEAPONS ) ) {
+               _turnStage++;
+       
 	} else if (_turnStage == CALCULATE_ATTACK) {
 		int overshoot;
 		int lastOvershoot = 0;    // avoid compiler warning
@@ -1665,7 +1240,7 @@ int PLAYER::computerControls (TANK *ctank)
 		int ydist = (int)(_targetY - ctank->y);
 
 		ctank->smallestOvershoot = _global->screenWidth;
-		if (ctank->howBuried () > 135) {
+		if (ctank->howBuried () > BURIED_LEVEL) {
 			_targetAngle = 180;
 			_targetPower = MAX_POWER / 10;
 		} else {
@@ -1685,14 +1260,8 @@ int PLAYER::computerControls (TANK *ctank)
 							// There could be a
 							//   more useful test?
 							_targetAngle = (rand () % 180) + 90;
-							_targetPower = MAX_POWER / 2;
+                                                        _targetPower = MAX_POWER / 2;
 						} else {
-							/*computerSelectItem (ctank);
-							selectTarget (&_targetX, &_targetY);
-							xdist = (int)(_targetX - ctank->x);
-							ydist = (int)(_targetY - ctank->y);
-							*/
-
 							_targetAngle = calculateAngle (xdist, ydist);
 							_targetAngle = adjustAngleGivenClearance (ctank, xdist, ydist, 150);
 							_targetPower = calculatePowerGivenAngle (xdist, ydist, _targetAngle);
@@ -1713,11 +1282,6 @@ int PLAYER::computerControls (TANK *ctank)
 					_targetPower = ctank->bestPower;
 				}
 			}
-			/*if (_targetAngle == -1) {
-				computerSelectItem (ctank);
-				selectTarget (&_targetX, &_targetY);
-				_turnStage--;
-			}*/
 		}
 		_turnStage++;
 
@@ -1755,6 +1319,8 @@ int PLAYER::computerControls (TANK *ctank)
 
 int PLAYER::humanControls (TANK *ctank)
 {
+     int moved = 0;
+
 	if (ctank) {
 		if (!_env->mouseclock && mouse_b & 1 && mouse_x >= 250
 			&& mouse_x < 378 && mouse_y >= 11 && mouse_y < 19) {
@@ -1780,7 +1346,8 @@ int PLAYER::humanControls (TANK *ctank)
 	}
 	
 	//Keyboard control
-	if (!_env->stage) {
+	if ( _env->stage == STAGE_AIM)    
+        {
 		if (ctank) {
 			if (key[KEY_LEFT] && !ctrlUsedUp && ctank->a < 270) {
 				ctank->a++;
@@ -1827,7 +1394,7 @@ int PLAYER::humanControls (TANK *ctank)
 		}
 	}
 	if (k && !fi) {
-		if (!_env->stage) {
+		if ( _env->stage == STAGE_AIM) {
 			if (ctank) {
 				if (k >> 8 == KEY_N) {
 					ctank->a = 180;
@@ -1879,6 +1446,14 @@ int PLAYER::humanControls (TANK *ctank)
                                     return (computerControls (ctank));
                                 }
 
+                                // move the tank
+                                if ((k >> 8 == KEY_COMMA) )
+                                   moved = ctank->Move_Tank(DIR_LEFT);
+                                if ((k >> 8 == KEY_STOP) )
+                                   moved = ctank->Move_Tank(DIR_RIGHT);
+                                if (moved)
+                                   _global->updateMenu = 1;
+ 
 				if ((k >> 8 == KEY_SPACE) &&
 					(((ctank->cw < WEAPONS) && (ctank->player->nm[ctank->cw] > 0)) ||
 						((ctank->cw < THINGS) && (ctank->player->ni[ctank->cw - WEAPONS] > 0)))) {
@@ -1888,7 +1463,7 @@ int PLAYER::humanControls (TANK *ctank)
 				}
 			}
 		}
-		if ((_env->stage == 3) && (k >> 8 == KEY_ENTER || k >> 8 == KEY_ESC || k >> 8 == KEY_SPACE))
+		if ((_env->stage == STAGE_ENDGAME) && (k >> 8 == KEY_ENTER || k >> 8 == KEY_ESC || k >> 8 == KEY_SPACE))
 			return (-1);
 	}
 	return (0);
@@ -1911,3 +1486,206 @@ char *PLAYER::Get_Team_Name()
     return team_name;
 }
 
+
+
+// Pick a weapon to fire at random.
+// The weapon number is returned. If no other
+// weapon is in inventory, then 0 - small missile is
+// used.
+int PLAYER::Select_Random_Weapon()
+{
+     int index;
+     int num_weapons = 0;
+     int random_weapon;
+
+     // count number of different weapons we have
+     for (index = 1; index < WEAPONS; index++)
+     {
+         if ( nm[index] )
+           num_weapons++;
+     }
+
+    // we have no weapons, use default
+    if ( num_weapons == 0 )
+       return 0;
+
+    // pick a random offset from the bottom of the list
+    random_weapon = (rand() % num_weapons) + 1;
+
+    index = WEAPONS - 1;
+    while ( (index) && (random_weapon) )
+    {
+        if ( nm[index] )
+            random_weapon--;
+        if (random_weapon)
+            index--; 
+    }
+
+    return index;
+}
+
+
+
+// This function selects a random item for
+// the AI to use.
+// This item to use is returned. If no
+// item can be found, then the function
+// returns 0
+int PLAYER::Select_Random_Item()
+{
+   int index, item_num;
+   int num_items = 0;
+   int random_item;
+   
+   // count the items we have
+   for (index = WEAPONS; index < THINGS; index++)
+   {
+       item_num = index - WEAPONS;
+       if ( (ni[item_num] ) && ( item[item_num].selectable) )
+           num_items++;
+   }    // end of counting items
+
+   if (! num_items)
+      return 0;
+
+    // if we have an item, there is still a 75% chance
+    // that we may not use it
+    if (! (rand() % 4) )
+         return 0;
+
+    // pick a random offset from the bottom of the list
+    random_item = (rand() % num_items) + 1;
+
+    index = THINGS - 1;
+    while ( (index) && (random_item) )
+    {
+        if ( ( ni[index - WEAPONS] ) && ( item[index - WEAPONS].selectable) )
+            random_item--;
+        if (random_item)
+            index--;
+    }
+
+    // do not use teleport without a parachute
+    item_num = index - WEAPONS;
+    if ( ( item_num == ITEM_TELEPORT) && (! ni[ITEM_PARACHUTE] ) )
+       index = 0;
+
+    // do not self destruct
+    if ( (item_num == ITEM_VENGEANCE) || (item_num == ITEM_DYING_WRATH) ||
+         (item_num ==  ITEM_FATAL_FURY) )
+       index = 0;
+
+    return index;
+}
+
+
+
+
+// This function selects which X and Y position to shoot
+// at. The following factors are taken into consideration.
+//
+// 1. Are we on the same team?
+// 2. Is the target stronger than I am?
+// 
+double PLAYER::Select_Target(int *target_X, int *target_Y)
+{
+    int random_target;
+    int attempts = 0, max_attempts;
+    TANK *best_target = NULL;
+    int current_score, best_score = 0;
+    int tank_count, object_count;
+    TANK *current_tank;
+
+    // find out how many tries we have to find a good target
+    switch ( (int) type )
+    {
+      case USELESS_PLAYER: max_attempts = 5; break;
+      case GUESSER_PLAYER: max_attempts = 10; break;
+      case RANGEFINDER_PLAYER: max_attempts = 20; break;
+      case TARGETTER_PLAYER: max_attempts = 30; break;
+      case DEADLY_PLAYER:
+      case PART_TIME_BOT:
+      default:    max_attempts = 50; break;
+    }
+
+
+    // who do we want to shoot at?
+    while ( attempts < max_attempts )
+    {
+        // select random tank for target
+        random_target = rand() % _global->numPlayers;
+
+        tank_count = 0;
+        object_count = 0;
+        current_tank = (TANK*) _env->getNextOfClass (TANK_CLASS, &object_count);
+        while ( tank_count < random_target )
+        {
+            tank_count++;
+            object_count++;
+            current_tank = (TANK*) _env->getNextOfClass (TANK_CLASS, &object_count);
+        }
+    
+        if (current_tank)
+        {    
+          // assign score to target
+          // is the target alive
+          if (current_tank->l > 0)
+            current_score = 500;
+          else
+            current_score -= 500;
+
+          // is the target weaker than we are
+          if ( (tank->l + tank->sh) > (current_tank->l + current_tank->sh) )
+             current_score += 200;
+
+          // check to see if we are on the same team
+          if ( ( team == TEAM_JEDI) || (team == TEAM_SITH ) )
+          {
+               if ( team == current_tank->player->team )
+                  current_score -= 100;
+               else
+                  current_score += 100;
+          }
+      
+          // try not to shoot ourselves 
+          if ( current_tank->player == this )
+             current_score -= 100;
+
+          // do we have a grudge against the target
+          if ( current_tank->player == revenge )
+             current_score += 200;
+
+          // prefer targets further away when violent death is on
+          if (_global->violent_death)
+          {
+              int distance;
+              distance = (int) fabs(tank->x - current_tank->x);
+              if (distance > _global->halfWidth)
+                 current_score += 100;
+          }
+
+          // decide if this target is better than others
+          if (current_score > best_score)
+          {
+              best_score = current_score;
+              best_target = current_tank;
+          }
+        }     // end of if we have a valid tank
+          attempts++;
+    }
+
+    // assign target values
+    if (best_target)
+    {
+        *target_X = (int) best_target->x;
+        *target_Y = (int) best_target->y;
+    }
+    else       // no target, pick a random one
+    {
+        *target_X = rand() % _global->screenWidth + 1;
+        *target_Y = _env->surface[ *target_X ];
+    }
+
+    return best_score;
+}
+
