@@ -52,6 +52,8 @@ enum turnStages {
 
 #define	NUM_ROULETTE_SLOTS (THINGS * 100)
 #define	MAX_WEAP_PROBABILITY	10000
+#define BURIED_LEVEL 135
+
 
 class TANK;
 class PLAYER {
@@ -67,12 +69,11 @@ class PLAYER {
 	int	_overshoot;
 	int	_weaponPreference[THINGS];
 	int	_rouletteWheel[NUM_ROULETTE_SLOTS];
-	double	*_targetMatrix;
 	int	_targetX;
 	int	_targetY;
 
 	public:
-	double	type;
+	double	type, type_saved;
 	double preftype;
 	char* preftypeText[ALWAYS_PREF + 1];
         char *tank_type[2];
@@ -92,6 +93,7 @@ class PLAYER {
 	double	errorMultiplier;
 	int	score;
 	double	played, won;
+        int     kills, killed;
 	double	selected;
 	int	money;
 	double	damageMultiplier;
@@ -126,11 +128,10 @@ class PLAYER {
 	int	calculateAngle (int dx, int dy);
 	int	calculateDirectAngle (int dx, int dy);
 	int	adjustAngleGivenClearance (TANK *ctank, int xdist, int ydist, int clearance);
-	int	addTankDamageToMatrix (TANK *ctank, TANK *ltank, int weapNum);
-	void	calcDamageMatrix (TANK *ctank, int weapNum);
 	double	calcDefenseValue (TANK *ctank, TANK *ltank);
 	double	selectTarget (); // select x to aim for
-	double	selectTarget (int *targetXCoord, int *targetYCoord); // select x to aim for
+	double	selectTarget (int *targetXCoord, int *targetYCoord); // select x to aim for 
+        double  Select_Target (int *target_X, int *target_Y); // select x to aim for
 	int	computerSelectItem (TANK *ctank); // Choose weapon to fire
 	int	chooseItemToBuy ();
 	void	generatePreferences ();
@@ -139,12 +140,12 @@ class PLAYER {
 	char	*selectRevengePhrase (double scale);
 	char	*selectGloatPhrase (double scale);
         char    *selectSuicidePhrase (double scale);
-	int	saveToFile (FILE *file);
-	int	loadFromFile (FILE *file);
         int     saveToFile_Text (FILE *file);
         int     loadFromFile_Text (FILE *file);
 	void	initMenuDesc ();
         char    *Get_Team_Name();
+        int     Select_Random_Weapon();
+        int     Select_Random_Item();
 };
 
 #endif
