@@ -28,6 +28,7 @@
 #include "beam.h"
 #include "tank.h"
 
+
 TANK::TANK ()
 {
 	_align = LEFT;
@@ -39,6 +40,23 @@ TANK::TANK ()
 
 TANK::~TANK ()
 {
+        int random_item;
+
+        if (_global->violent_death)
+        {
+            random_item = rand() % VIOLENT_CHANCE;
+            if ( ( random_item > _global->violent_death ) && (random_item <= VIOLENT_DEATH_HEAVY) )
+               random_item = (int) _global->violent_death;
+
+          switch (random_item)
+          {
+             case VIOLENT_DEATH_LIGHT: player->ni[ITEM_VENGEANCE] += 1; break;
+             case VIOLENT_DEATH_MEDIUM: player->ni[ITEM_DYING_WRATH] += 1; break;
+             case VIOLENT_DEATH_HEAVY: player->ni[ITEM_FATAL_FURY] += 1; break;
+          }     //end of switch
+
+        }
+
 	if (player->ni[ITEM_FATAL_FURY] > 0) {
 		int numLaunch = (int)item[ITEM_FATAL_FURY].vals[SELFD_NUMBER];
 		cw = (int)item[ITEM_FATAL_FURY].vals[SELFD_TYPE];
@@ -174,6 +192,8 @@ void TANK::newRound ()
 	healthText->set_text (buf);
         // delay_fall = GRAVITY_DELAY;
         delay_fall = (int) _env->landSlideDelay * 100;
+        fire_another_shot = 0;
+        shots_fired = 0;
 
 }
 
@@ -201,10 +221,13 @@ void TANK::applyDamage ()
         {
 	    damage = sh + l;
             killed = true;
+            player->killed++;
         }
 	sh -= (int)damage;
 	if (creditTo) {
 		if (player != creditTo) {	//enemy hit ++
+                        if ( killed )
+                           creditTo->kills++;
 			creditTo->money += (int)(damage * _global->scoreHitUnit);
                         // this tank is destroyed, the attacker gloats
                         if ( (killed) && (! creditTo->gloating) )
@@ -477,6 +500,14 @@ void TANK::draw (BITMAP *dest, int healthOffset)
 {
 	int turretAngle;
 
+        // check for foggy weather
+        if ( ( _env->fog ) && ( _global->currTank != this ) )
+        {
+           addUpdateArea ((int)(x - TANKWIDTH) - 3, (int)y - 25, 35, 46);
+	   requireUpdate ();
+           return;
+        }
+
         // get bitmap for tank
         if (player)
         {
@@ -581,10 +612,15 @@ void TANK::simActivateCurrentSelection ()
        if (_global->turntype != TURN_SIMUL)
        {
                activateCurrentSelection();
+               if (fire_another_shot)
+                   fire_another_shot--;
        } else {
                _env->stage = 1;
        }
 
+       // allow naturals to happen again
+       _env->naturals_since_last_shot = 0;
+
        // apply repairs
        l += repair_rate;
        if (l > maxLife)
@@ -646,6 +682,10 @@ void TANK::activateCurrentSelection ()
 				newmis->physics = 0;
 				newmis->age = 0;
 				newmis->player = player;
+                                // set up volley
+                                if (! fire_another_shot)
+                                   fire_another_shot = weapon[cw].delay * VOLLY_DELAY;
+
 				if (player->ni[ITEM_DIMPLEP]) {
 					player->ni[ITEM_DIMPLEP]--;
 					newmis->drag *= item[ITEM_DIMPLEP].vals[0];
@@ -746,6 +786,12 @@ void TANK::activateCurrentSelection ()
           player->changed_weapon = true;
        }
 
+       shots_fired++;
+
+       // if not performing a volly and the player is AI then randomly select next weapon
+       // else if ( ( player->type > HUMAN_PLAYER ) && (! fire_another_shot) )
+       //    cw = player->Select_Random_Weapon();
+
 }
 
 
@@ -878,3 +924,48 @@ int TANK::Get_Repair_Rate()
      return repair_units;
 }
 
+
+
+/*
+This function tries to move the tank either
+left or right one unit. The direction is passed
+in.
+The function returns TRUE if the tank is moved and
+FALSE if something is in the way or the
+tank cannot be moved for some reason.
+-- Jesse
+*/
+int TANK::Move_Tank(int direction)
+{
+   int pixel;
+   int destination_x;
+
+   // do we have fuel?
+   if ( player->ni[ITEM_FUEL] < 1 )
+      return FALSE;
+
+   // see where we want to go
+   if (direction == DIR_RIGHT)
+      destination_x = (int) (x + TANKWIDTH - 2);
+   else
+      destination_x = (int) (x - (TANKWIDTH + 1) );
+
+   if (_env->landType != LANDTYPE_NONE)
+   {
+      // check for something in the way
+      pixel = getpixel (_env->terrain, destination_x, (int) (y + (TANKHEIGHT / 2)) );
+      if (pixel != PINK)
+        return FALSE;
+   }
+
+   // move tank
+   if (direction == DIR_RIGHT)
+      x++;
+   else
+      x--;
+
+   player->ni[ITEM_FUEL]--;
+
+   return TRUE;
+}
+
