@@ -25,6 +25,11 @@
 
 #define	TELETIME	120
 
+#define DIR_RIGHT 1
+#define DIR_LEFT 2
+
+#define VIOLENT_CHANCE 6
+
 
 class PLAYER;
 class EXPLOSION;
@@ -56,6 +61,8 @@ class TANK: public PHYSICAL_OBJECT
         int use_tank_bitmap, use_turret_bitmap;
         double turret_x, turret_y;
         int delay_fall;         // time the tank will hover
+        int fire_another_shot;
+        int shots_fired;
 
 	TANK ();
 	~TANK ();
@@ -82,6 +89,7 @@ class TANK: public PHYSICAL_OBJECT
 	void	framelyAccounting ();
         int     tank_on_tank(GLOBALDATA *gd); //is this tank on top of another?
         int     Get_Repair_Rate();    // how many units a tank will repair itself per round
+        int     Move_Tank(int direction);
 };
 
 #endif
